#include "DistrhoPlugin.hpp"

START_NAMESPACE_DISTRHO

class TabPlugin : public Plugin
{
public:
	TabPlugin()
		: Plugin(kParameterCount, 0, 0),
		sampleRate(getSampleRate())
	{
		// clear all parameters
		std::memset(fParameters, 0, sizeof(float) * kParameterCount);

	}

protected:
	const char* getLabel() const override { return "yaw-tab"; }
	const char* getDescription() const override { return "Drawing tablet synth UI"; }
	const char* getMaker() const override { return "yaw-audio"; }
	const char* getHomePage() const override { return "https://yaw.man/plugins/yaw-tab"; }
	const char* getLicense() const override { return "ISC"; }
	uint32_t    getVersion() const override { return d_version(1, 0, 0); }
	int64_t     getUniqueId() const override { return d_cconst('y', 'w', 't', 'b'); }


	void initParameter(uint32_t index, Parameter& parameter) override
	{
		parameter.hints = kParameterIsAutomable;
		parameter.ranges.def = 0.0f;
		parameter.ranges.min = 0.0f;
		parameter.ranges.max = 16777216.0f;

		switch (index)
		{
		case kParameterTabletX:
			parameter.name = "x";
			parameter.symbol = "x";
			parameter.ranges.min = 0.0f;
			parameter.ranges.max = 999.0f;
			break;
		case kParameterTabletY:
			parameter.name = "y";
			parameter.symbol = "y";
			parameter.ranges.min = 0.0f;
			parameter.ranges.max = 999.0f;
			break;
		case kParameterTabletZ:
			parameter.name = "z";
			parameter.symbol = "z";
			parameter.ranges.min = 0.0f;
			parameter.ranges.max = 999.0f;
			break;
		case kParameterTabletPressure:
			parameter.name = "p";
			parameter.symbol = "p";
			parameter.ranges.min = 0.0f;
			parameter.ranges.max = 999.0f;
			break;
		case kParameterTime:
			parameter.name = "t";
			parameter.symbol = "t";
			parameter.ranges.min = 0.0f;
			parameter.ranges.max = 999.0f;
			parameter.hints |= kParameterIsOutput;
			break;
		}
	}

	void sampleRateChanged(double newRate) override
	{
		sampleRate = newRate;
	}

	float getParameterValue(uint32_t index) const override
	{
		return fParameters[index];

	}

	void setParameterValue(uint32_t idx, float val) override
	{
		fParameters[idx] = val;
		switch (idx) {
		case kParameterTabletX:
			period = 0.02f * (0.001f * val) * sampleRate;
			break;
		case kParameterTabletY:
			break;
		case kParameterTabletZ:
			break;
		case kParameterTabletPressure:
			volume = 0.001f * val;
			break;
		}
	}

	void run(const float** inputs, float** outputs, uint32_t frames) override
	{

		for (uint32_t i = 0; i < frames; ++i) {
			counter++;
			if (counter > period) {
				outputs[0][i] = outputs[1][i] = volume;
				counter = 0;
			}
			else {
				outputs[0][i] = outputs[1][i] = 0.0f;
			}
		}

		// Force UI to repaint via parameter change callback.
		const TimePosition& timePos(getTimePosition());
		fParameters[kParameterTime] = timePos.bbt.tick;
	}

private:
	float fParameters[kParameterCount];
	float period = 0.f;
	float counter = 0.f;
	float volume = 0.f;
	float sampleRate;

	DISTRHO_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR(TabPlugin)
};

Plugin* createPlugin()
{
	return new TabPlugin();
}

END_NAMESPACE_DISTRHO
